@@ -1,16 +1,19 @@
 import tensorflow as tf
 import keras
+from importlib import resources
 
-def tensorflow_predict(model: keras.Model, input_data: dict) -> dict[str, float]:
+def tensorflow_predict(input_data: dict) -> dict[str, float]:
     """Make predictions using a TensorFlow model and input data.
     
     Args:
-        model (keras.Model): The TensorFlow model to use for predictions.
         input_data (dict): Input data for making predictions.
     
     Returns:
         dict[str, float]: Dictionary of predictions made by the model. prediction_normal and prediction_portscan
     """
+    # Load the TensorFlow model
+    model = load_model()
+
     # Convert input data to tensors
     input_dict = {name: tf.convert_to_tensor([value]) for name, value in input_data.items()}
 
@@ -29,4 +32,13 @@ def tensorflow_predict(model: keras.Model, input_data: dict) -> dict[str, float]
     except Exception:
         raise RuntimeError("Failed to process predictions from the TensorFlow model.")
 
-    return output_dict
\ No newline at end of file
+    return output_dict
+
+
+# Load TensorFlow model from package resources
+def load_model():
+    """Load TensorFlow model from package resources."""
+    # Use with to get a temporary path to the model file
+    with resources.path('gresecml.ml_models', 'tensorflow.keras') as model_path:
+        model = keras.models.load_model(model_path)
+    return model
\ No newline at end of file
